@@ -1,7 +1,7 @@
 package IIIF::ImageAPI;
 use 5.014001;
 
-our $VERSION = "0.01";
+our $VERSION = "0.02";
 
 use parent 'Plack::Component';
 
@@ -56,7 +56,7 @@ sub response {
     # allow abbreviated requests, redirect to full form
     my $request = eval { IIIF::Request->new($local) };
     if ($@) {
-        return error_response( 400, ( split( ":", $@ ) )[0] );
+        return error_response( 400, ( split( " at ", $@ ) )[0] );
     }
 
     $request->{format} = $request->{format} // $file->{format};
@@ -79,8 +79,19 @@ sub response {
     if ( -r $cache_file ) {
         return image_response($cache_file);
     }
-    elsif ( convert( $request, $file->{path}, $cache_file ) ) {
-        return image_response($cache_file);
+    else {
+
+        # TODO: only get image dimensions once and only if actually needed
+        my $info = info( $file->{path} );
+        if ( !$request->fits($info) ) {
+            return error_response( 400,
+                "Invalid IIIF Image API Request: region or size out of bounds"
+            );
+        }
+
+        if ( convert( $request, $file->{path}, $cache_file ) ) {
+            convert return image_response($cache_file);
+        }
     }
 
     error_response( 500, "Conversion failed" );
