@@ -1,11 +1,11 @@
 package IIIF::Magick;
 use 5.014001;
 
-our $VERSION = "0.01";
+our $VERSION = "0.02";
 
 use Exporter;
 our @ISA    = qw(Exporter);
-our @EXPORT = qw(info available convert);
+our @EXPORT = qw(info available convert args convert_command);
 
 use IPC::Cmd qw(can_run);
 use List::Util qw(min);
@@ -65,17 +65,23 @@ sub args {
     }
     elsif ( $req->{size_px} ) {
         my ( $x, $y ) = @{ $req->{size_px} };
+
+        # TODO: respect upscale
+
         if ( $x && $y ) {
-            push @args, '-resize', "${x}x$y!";
+            if ( $req->{ratio} ) {
+                push @args, '-resize', "${x}x$y";
+            }
+            else {
+                push @args, '-resize', "${x}x$y!";
+            }
         }
         elsif ( $x && !$y ) {
             push @args, '-resize', "${x}";
         }
         elsif ( !$x && $y ) {
-            push @args, '-resize', "x${x}";
+            push @args, '-resize', "x${y}";
         }
-
-        # TODO: upscale, limit, ...
     }
 
     # apply rotation
@@ -92,19 +98,13 @@ sub args {
         push @args, qw(-monochrome -colors 2);
     }
 
-    if (@args) {
-        say STDERR "\n", join ' ', map { shell_quote($_) } @args;
-    }
+    # if (@args) {
+    #    say STDERR "\n", join ' ', map { shell_quote($_) } @args;
+    # }
 
     return @args, $file;
 }
 
-sub convert {
-    my ( $req, $in, $out ) = @_;
-    run( 'convert', args( $req, $in ), $out );
-    return !$?;
-}
-
 # adopted from <https://metacpan.org/release/ShellQuote-Any-Tiny>
 sub shell_quote {
     my $arg = shift;
@@ -118,7 +118,7 @@ sub shell_quote {
         return qq("$arg");
     }
     else {
-        if ( $arg =~ qr{\A[\w,_+/-]+\z} ) {
+        if ( $arg =~ qr{\A[\w,_+/.-]+\z} ) {
             return $arg;
         }
         $arg =~ s/'/'"'"'/g;
@@ -126,6 +126,18 @@ sub shell_quote {
     }
 }
 
+sub convert {
+    my ( $req, $in, $out ) = @_;
+    run( 'convert', args( $req, $in ), $out );
+    return !$?;
+}
+
+sub convert_command {
+    my ( $req, $in, $out ) = @_;
+    my @args = args( $req, $in );
+    return join ' ', 'convert', @args, $out;
+}
+
 sub run {
     unshift @_, "magick" if can_run("magick");
     my $command = join ' ', map &shell_quote, @_;
@@ -135,11 +147,24 @@ sub run {
 1;
 __END__
 
+=head1 NAME
+
+IIIF::Magick - transform image with IIIF Image API Request using Image Magick
+
 =head1 SYNOPSIS
 
-    use IIIF::Magick qw(info);
+    use IIIF::Magick qw(info convert);
 
     my $info = info($file, profile => "level0", id => "...") ;
+    
+    convert( $request, $file, "target.png" );
+
+=head1 DESCRIPTION
+
+This module maps L<IIIF ImageAPI|https://iiif.io/api/image/3.0/> request
+parameters to L<ImageMagick|https://www.imagemagick.org/> command line
+arguments. See L<i3f> (command line) and L<IIIF::ImageAPI> (web service)
+for applications that make use of it.
 
 =head1 FUNCTIONS
 
@@ -153,4 +178,18 @@ Returns L<image information|https://iiif.io/api/image/3.0/#5-image-information>
 object with fields C<@context>, C<type>, C<protocol>, C<width>, and C<height>.
 Fields C<id> and C<profile> must be added for full IIIF compliance.
 
+=head2 convert( $request, $file, $output )
+
+Convert an image file as specified with a L<IIIF::Request> into an output file.
+Returns true on success.
+
+=head2 args( $request, $file )
+
+Get the list of command line arguments to C<convert> to transform an image file
+as specified via a L<IIIF::Request>.
+
+=head2 command( $command, @arguments )
+
+Get call to an ImageMagick command (e.g. C<convert>) with shell-quoted arguments.
+
 =cut
