@@ -1,9 +1,10 @@
 package IIIF::Request;
 use 5.014001;
 
-our $VERSION = "0.01";
+our $VERSION = "0.02";
 
 use Plack::Util::Accessor qw(region size rotation quality format);
+use Carp qw(croak);
 
 our $XY = qr{[0-9]+};         # non-negative integer
 our $WH = qr{[1-9][0-9]*};    # positive integer
@@ -23,7 +24,7 @@ sub new {
 
     my ( $rotation, $mirror, $degree, $quality, $format );
     my ( $region, $region_pct, $region_px );
-    my ( $size, $upscale, $size_px, $size_pct, $limit );
+    my ( $size, $upscale, $size_px, $size_pct, $ratio );
 
     my @parts = split '/', $path;
 
@@ -40,22 +41,18 @@ sub new {
     if ( @parts && $parts[0] =~ /^$SIZE$/ ) {
         $size    = shift @parts;
         $upscale = $1;
-        $limit   = $6;
-        $size_px = $4 // $5 // $7;
+        $ratio   = $7;
+        $size_px = [ split ',', $5 // $6 // $8 ] if $5 // $6 // $8;
 
         if ( defined $3 ) {
             $size_pct = 1 * $3;
-            $size = $upscale ? "^pct:$size_pct" : "pct:$size_pct";
-        }
-
-        if ( !$upscale ) {
-            if ( defined $size_pct ) {
-                die "invalid percentage in IIIF API request: $path"
-                  if $size_pct == 0.0 || $size_pct > 100.0;
+            if ($upscale) {
+                $size = "^pct:$size_pct";
             }
-            elsif ( defined $size_px ) {
-
-                # TODO: error if greater than extracted region
+            else {
+                error("disallowed percentage value")
+                  if $size_pct == 0.0 || $size_pct > 100.0;
+                $size = "pct:$size_pct";
             }
         }
     }
@@ -75,7 +72,8 @@ sub new {
         shift @parts;
     }
 
-    die "invalid IIIF API request: $path" if @parts;
+    error( "failed to parse '" . join( '/', '', @parts ) . "'" )
+      if @parts;
 
     bless {
         region => $region // 'full',
@@ -85,7 +83,7 @@ sub new {
         upscale    => $upscale,
         size_pct   => $size_pct,
         size_px    => $size_px,
-        limit      => $limit,
+        ratio      => $ratio,
         rotation   => $rotation // '0',
         mirror     => $mirror,
         degree     => $degree,
@@ -94,6 +92,28 @@ sub new {
     }, $class;
 }
 
+sub error {
+    croak "Invalid IIIF Image API Request: $_[0]";
+}
+
+sub fits {
+    my ( $self, $info ) = @_;
+
+    # region outside the bounds of image dimensions?
+    if ( $self->{region_px} ) {
+        my ( $x, $y, $w, $h ) = @{ $self->{region_px} };
+        return ( $x < $info->{width} && $y < $info->{height} );
+    }
+
+    # size larger than region (unless upscale)
+    if ( !$self->{upscale} ) {
+
+        # TODO
+    }
+
+    return 1;
+}
+
 sub is_default {
     my ($self) = @_;
 
@@ -122,13 +142,15 @@ IIIF::Request - IIIF Image API request object
 
 =head1 DESCRIPTION
 
-Stores the part of an IIIF Image API URL after C<{identifier}>:
+Stores parts of an L<IIIF ImageAPI|https://iiif.io/api/image/3.0/> after
+C<{identifier}>:
 
     {region}/{size}/{rotation}/{quality}.{format}
 
 In contrast to the IIIF Image API Specification, all parts are optional.
-Omitted parts are set to their default value except for C<format> which may be
-undefined. In addition, the following fields may be set:
+Omitted parts are set to their default value, except for C<format> which may be
+undefined. In addition, the following fields are set if deriveable form the
+request:
 
 =over
 
@@ -142,7 +164,7 @@ undefined. In addition, the following fields may be set:
 
 =item size_px
 
-=item limit
+=item ratio
 
 =item mirror
 
@@ -150,19 +172,33 @@ undefined. In addition, the following fields may be set:
 
 =back
 
+Request parsing of percentage and degree values is also more forgiving than the
+specification.
+
 =head1 METHODS
 
 =head2 new( [ $request ] )
 
 Parses a request string. It's ok to only include selected image manipulations.
+Will raise an error on invalid requests. The request is parsed independent from
+a specific image so regions and sizes outside of the image bound are not
+detected as invalid.
+
+=head2 fits( $width, $height )
+
+Check whether the request would be valid for an image of given width and height.
 
 =head2 as_string
 
-Returns the full request string.
+Returns the full request string. Percentage values and degrees are normalized.
 
 =head2 is_default
 
 Returns whether the request (without format) is the default request
 C<full/max/0/default> to get an unmodified image.
 
+=head2 error
+
+Raise an "Invalid IIIF Image API Request" error. Can also be used as function.
+
 =cut
